@@ -1,5 +1,3 @@
-# @file InsertTable.R
-#
 # Copyright 2025 Observational Health Data Sciences and Informatics
 #
 # This file is part of DatabaseConnector
@@ -16,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-getSqlDataTypes <- function(column) {
+getSqlDataTypes <- function(column, dbms) { 
   if (is.integer(column)) {
     return("INTEGER")
   } else if (is(column, "POSIXct") | is(column, "POSIXt")) {
@@ -27,6 +25,13 @@ getSqlDataTypes <- function(column) {
     return("BIGINT")
   } else if (is.numeric(column)) {
     return("FLOAT")
+  } else if (is.logical(column)) {
+    return(switch(
+      dbms,
+      "sql server" = "BIT",
+      "oracle" = "NUMBER(1)", # could also consider `NUMBER(1)` possibly with constraint `COLNAME NUMBER(1) CHECK (COLNAME IN (0, 1))`
+      "BOOLEAN"
+    ))
   } else {
     if (is.factor(column)) {
       maxLength <-
@@ -182,7 +187,6 @@ insertTable <- function(connection,
                         dropTableIfExists = TRUE,
                         createTable = TRUE,
                         tempTable = FALSE,
-                        oracleTempSchema = NULL,
                         tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
                         bulkLoad = Sys.getenv("DATABASE_CONNECTOR_BULK_UPLOAD"),
                         useMppBulkLoad = Sys.getenv("USE_MPP_BULK_LOAD"),
@@ -192,19 +196,18 @@ insertTable <- function(connection,
 }
 
 #' @export
-insertTable.default <- function(connection,
-                                databaseSchema = NULL,
-                                tableName,
-                                data,
-                                dropTableIfExists = TRUE,
-                                createTable = TRUE,
-                                tempTable = FALSE,
-                                oracleTempSchema = NULL,
-                                tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
-                                bulkLoad = Sys.getenv("DATABASE_CONNECTOR_BULK_UPLOAD"),
-                                useMppBulkLoad = Sys.getenv("USE_MPP_BULK_LOAD"),
-                                progressBar = FALSE,
-                                camelCaseToSnakeCase = FALSE) {
+insertTable.DatabaseConnectorJdbcConnection <- function(connection,
+                                                       databaseSchema = NULL,
+                                                       tableName,
+                                                       data,
+                                                       dropTableIfExists = TRUE,
+                                                       createTable = TRUE,
+                                                       tempTable = FALSE,
+                                                       tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
+                                                       bulkLoad = Sys.getenv("DATABASE_CONNECTOR_BULK_UPLOAD"),
+                                                       useMppBulkLoad = Sys.getenv("USE_MPP_BULK_LOAD"),
+                                                       progressBar = FALSE,
+                                                       camelCaseToSnakeCase = FALSE) {
   if (is(connection, "Pool")) {
     connection <- pool::poolCheckout(connection)
     on.exit(pool::poolReturn(connection))
@@ -218,13 +221,7 @@ insertTable.default <- function(connection,
     bulkLoad <- useMppBulkLoad
   }
   bulkLoad <- (!is.null(bulkLoad) && bulkLoad == "TRUE")
-  if (!is.null(oracleTempSchema) && oracleTempSchema != "") {
-    warn("The 'oracleTempSchema' argument is deprecated. Use 'tempEmulationSchema' instead.",
-         .frequency = "regularly",
-         .frequency_id = "oracleTempSchema"
-    )
-    tempEmulationSchema <- oracleTempSchema
-  }
+  
   if (is_installed("Andromeda") && Andromeda::isAndromedaTable(data)) {
     warn("Batch-wise uploading of Andromeda tables currently not supported. Loading entire table in memory.",
          .frequency = "regularly",
@@ -243,7 +240,13 @@ insertTable.default <- function(connection,
     createTable <- TRUE
   }
   if (tempTable & substr(tableName, 1, 1) != "#" & dbms != "redshift") {
-    tableName <- paste("#", tableName, sep = "")
+    if (startsWith(tableName, "\"")) {
+      tableName <- paste0("\"#", substr(tableName, 2, nchar(tableName)))
+    } else if (startsWith(tableName, "`")) {
+      tableName <- paste0("`#", substr(tableName, 2, nchar(tableName)))
+    } else {
+      tableName <- paste0("#", tableName)
+    }
   }
   if (!is.null(databaseSchema)) {
     tableName <- paste(databaseSchema, tableName, sep = ".")
@@ -264,7 +267,6 @@ insertTable.default <- function(connection,
       data <- as.data.frame(data)
     }
   }
-  data <- convertLogicalFields(data)
   isSqlReservedWord(c(tableName, colnames(data)), warn = TRUE)
   useBulkLoad <- (bulkLoad && dbms %in% c("hive", "redshift") && createTable) ||
     (bulkLoad && dbms %in% c("pdw", "postgresql") && !tempTable)
@@ -272,8 +274,7 @@ insertTable.default <- function(connection,
   if (dbms == "bigquery" && useCtasHack && is.null(tempEmulationSchema)) {
     abort("tempEmulationSchema is required to use insertTable with bigquery when inserting into a new table")
   }
-  
-  sqlDataTypes <- sapply(data, getSqlDataTypes)
+  sqlDataTypes <- sapply(data, getSqlDataTypes, dbms = dbms)
   sqlTableDefinition <- paste(.sql.qescape(names(data), TRUE), sqlDataTypes, collapse = ", ")
   sqlTableName <- .sql.qescape(tableName, TRUE, quote = "")
   sqlFieldNames <- paste(.sql.qescape(names(data), TRUE), collapse = ",")
@@ -291,7 +292,9 @@ insertTable.default <- function(connection,
   }
   
   if (createTable && !useCtasHack && !(bulkLoad && dbms == "hive")) {
+    
     sql <- paste("CREATE TABLE ", sqlTableName, " (", sqlTableDefinition, ");", sep = "")
+    
     renderTranslateExecuteSql(
       connection = connection,
       sql = sql,
@@ -378,6 +381,10 @@ insertTable.default <- function(connection,
             rJava::.jcall(batchedInsert, "V", "setDateTime", i, format(column, format="%Y-%m-%d %H:%M:%S"))
           } else if (is(column, "Date")) {
             rJava::.jcall(batchedInsert, "V", "setDate", i, as.character(column))
+          } else  if (is.logical(column)) {
+            # encode column as -1 (NA), 1 (TRUE), 0 (FALSE) to pass logical NAs into Java 
+            column <- vapply(as.integer(column), FUN = function(x) ifelse(is.na(x), -1L, x), FUN.VALUE = integer(1L))
+            rJava::.jcall(batchedInsert, "V", "setBoolean", i, column)
           } else {
             rJava::.jcall(batchedInsert, "V", "setString", i, as.character(column))
           }
@@ -406,19 +413,41 @@ insertTable.DatabaseConnectorDbiConnection <- function(connection,
                                                        dropTableIfExists = TRUE,
                                                        createTable = TRUE,
                                                        tempTable = FALSE,
-                                                       oracleTempSchema = NULL,
                                                        tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
                                                        bulkLoad = Sys.getenv("DATABASE_CONNECTOR_BULK_UPLOAD"),
                                                        useMppBulkLoad = Sys.getenv("USE_MPP_BULK_LOAD"),
                                                        progressBar = FALSE,
                                                        camelCaseToSnakeCase = FALSE) {
-  if (!is.null(oracleTempSchema) && oracleTempSchema != "") {
-    warn("The 'oracleTempSchema' argument is deprecated. Use 'tempEmulationSchema' instead.",
-         .frequency = "regularly",
-         .frequency_id = "oracleTempSchema"
-    )
-    tempEmulationSchema <- oracleTempSchema
-  }
+  insertTable(
+    connection = connection@dbiConnection,
+    databaseSchema = databaseSchema,
+    tableName = tableName,
+    data = data,
+    dropTableIfExists = dropTableIfExists,
+    createTable = createTable,
+    tempTable = tempTable,
+    tempEmulationSchema = tempEmulationSchema,
+    bulkLoad = bulkLoad,
+    useMppBulkLoad = useMppBulkLoad,
+    progressBar = progressBar,
+    camelCaseToSnakeCase = camelCaseToSnakeCase
+  )
+}
+
+#' @export
+insertTable.default <- function(connection,
+                                databaseSchema = NULL,
+                                tableName,
+                                data,
+                                dropTableIfExists = TRUE,
+                                createTable = TRUE,
+                                tempTable = FALSE,
+                                tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
+                                bulkLoad = Sys.getenv("DATABASE_CONNECTOR_BULK_UPLOAD"),
+                                useMppBulkLoad = Sys.getenv("USE_MPP_BULK_LOAD"),
+                                progressBar = FALSE,
+                                camelCaseToSnakeCase = FALSE) {
+  
   if (camelCaseToSnakeCase) {
     colnames(data) <- SqlRender::camelCaseToSnakeCase(colnames(data))
   }
@@ -455,7 +484,6 @@ insertTable.DatabaseConnectorDbiConnection <- function(connection,
     }
     
   }
-  data <- convertLogicalFields(data)
   
   logTrace(sprintf("Inserting %d rows into table '%s' ", nrow(data), tableName))
   if (!is.null(databaseSchema)) {
@@ -472,7 +500,7 @@ insertTable.DatabaseConnectorDbiConnection <- function(connection,
   
   startTime <- Sys.time()
   DBI::dbWriteTable(
-    conn = connection@dbiConnection,
+    conn = connection,
     name = tableName,
     value = data,
     overwrite = dropTableIfExists,
@@ -483,15 +511,3 @@ insertTable.DatabaseConnectorDbiConnection <- function(connection,
   inform(paste("Inserting data took", signif(delta, 3), attr(delta, "units")))
   invisible(NULL)
 }
-
-convertLogicalFields <- function(data) {
-  for (i in 1:ncol(data)) {
-    column <- data[[i]]
-    if (is.logical(column)) {
-      warn(sprintf("Column '%s' is of type 'logical', but this is not supported by many DBMSs. Converting to numeric (1 = TRUE, 0 = FALSE)", 
-                   colnames(data)[i]))
-      data[, i] <- as.integer(column)
-    }
-  }
-  return(data)
-}
