@@ -20,8 +20,14 @@ for (testServer in testServers) {
       renderTranslateExecuteSql(connection, createSql)
       expect_equal(renderTranslateExecuteSql(connection, sql, runAsBatch = TRUE), c(1, 1, 0))
       renderTranslateExecuteSql(connection, createSql)
-      rowsAffected <- dbSendStatement(connection, sql)
-      expect_equal(dbGetRowsAffected(rowsAffected), 2)
+      translatedSql <- SqlRender::translate(sql, dbms(connection))
+      statements <- SqlRender::splitSql(translatedSql)
+      rowsAffected <- dbSendStatement(connection, statements[1])
+      expect_equal(dbGetRowsAffected(rowsAffected), 1)
+      rowsAffected <- dbSendStatement(connection, statements[2])
+      expect_equal(dbGetRowsAffected(rowsAffected), 1)
+      rowsAffected <- dbSendStatement(connection, statements[3])
+      expect_equal(dbGetRowsAffected(rowsAffected), 0)
       dbClearResult(rowsAffected)
     }
   })
